/*
Morning Monster Source
nickhardeman.com/blog/?p=213
The clock functionality is from http://7in7.tumblr.com/post/38066438/open-source-arduino-clock-project by Rob Faludi
The printing to the lcd screen is from this arduino forum post
http://www.arduino.cc/cgi-bin/yabb2/YaBB.pl?num=1235524565/0
*/

#include <SoftwareSerial.h>

// analog pins 
//#define hourPin 0
#define flexSensor 0
#define minutePin 2
#define hourPinA 3
#define minutePinA 4
//#define alarmPin 4
#define snoozePin 12


// digital pins 
#define rxPin 2
#define txPin 3

//#define rxrfPin 4
//#define txrfPin 5

#define speakerPin 5

#define alarmPin 8
#define hourPin 9

#define ledPin 13


int second = 45, minute = 59, hour = 11, weekday = 7; // declare time variables
int minuteA = 0, hourA = 12;

int minuteSn = 0, hourSn = 12;

boolean alarmOn = false;
byte soundAlarm = 0;
// track if the seq of the alarm is active //
byte alarmSeq = 0;
// the number of mins prior to the alarm time that blinds should go off //
int blindPrevTime = 30;

boolean minPressed = false, hourPressed = false, minPressedA = false, hourPressedA = false, alarmPressed = false, snoozePressed = false;

boolean settingTime = false, settingTimeA = false;
unsigned long lastSetA;

char snoozeTime = 1;
boolean snoozing = false;
// keep track of the last time that it was pressed //
unsigned long snoozeTracker = 0;


// speaker vars //
unsigned long speakerTime = 0;
boolean speakerHigh = false;

// flex vars //
int flexVal = 480;


// set up a new serial connection for communicating with RF receiver 
// (this also frees up Arduino's built in serial port for PC communication)
SoftwareSerial lcd =  SoftwareSerial(rxPin, txPin); 
//SoftwareSerial rfSerial =  SoftwareSerial(rxrfPin, txrfPin);


void setup() {
  
  pinMode(txPin, OUTPUT);
  pinMode(rxPin, INPUT);
  
  //pinMode(rxrfPin, INPUT);
  //pinMode(txrfPin, OUTPUT);
  
  pinMode(alarmPin, INPUT);
  pinMode(hourPin, INPUT);
  
  //rfSerial.begin(2400);
  lcdSetup();
  
  Serial.begin(2400);
  pinMode(ledPin, OUTPUT);
  pinMode(snoozePin, INPUT);
  digitalWrite(ledPin, HIGH);
  pinMode(speakerPin, OUTPUT);
  digitalWrite(speakerPin, LOW);
  
  delay(500);
  
}

void loop() {

  if (settingTime) {
    second = 0;
    //serialOutput(false);
    printTime(hour, minute, 1);
  } else if (settingTimeA) {
    //serialOutput(true);
    printTime(hourA, minuteA, 2);
  } else {
    if (millis() - lastSetA > 300) {
      advanceTime();
      checkAlarm();
    }
  }
  checkButtons(); // runs a function that checks the setting buttons
  
  // flex check //
  flexVal = (int)analogRead(flexSensor);
  //Serial.println(flexVal);
  if (flexVal >= 430) {
    sendData(15);
  } else if (flexVal <= 365) {
    sendData(5);
  } else {
    //rfSerial.print(0, BYTE);
    //Serial.print(0);
  }
}

// advance the clock time //
void advanceTime() {
    // (static variables are initialized once and keep their values between function calls)
    static unsigned long lastTick = 0; // set up a local variable to hold the last time we moved forward one second
    
    // move forward one second every 1000 milliseconds
    if (millis() - lastTick >= 1000) {
      lastTick = millis();
      second++;
      //serialOutput(false);
      printTime(hour, minute, 0);
    }

    // move forward one minute every 60 seconds
    if (second >= 60) {
      minute++;
      second = 0; // reset seconds to zero
    }

    // move forward one hour every 60 minutes
    if (minute >=60) {
      hour++;
      minute = 0; // reset minutes to zero
    }

    // move forward one weekday every 24 hours
    if (hour >= 24) {
      weekday++;
      hour = 0; // reset hours to zero
    }

    // reset weekdays on Saturday
    if (weekday >= 8) {
      weekday = 1;
    }
}

void checkAlarm() {
  if (alarmOn == false) {
    digitalWrite(speakerPin, LOW);
  }
  
  if (alarmOn == true && soundAlarm < 1 && snoozing == false) {
    if (hourA == hour) {
      if (minuteA == minute) {
        sendData(10);
        soundAlarm = 1;
      }
    }
  }
  // check for the snoozy time //
  if (snoozing == true && soundAlarm < 1) {
    if (hourSn == hour) {
      if (minuteSn == minute) {
        sendData(10);
        soundAlarm = 1;
      }
    }
  }
  
  if (soundAlarm > 0) {
    onAlarm();
  }
}

void onAlarm() {
  playSpeaker();
  sendData(10);
  if (alarmOn == false) {
    soundAlarm = 0;
    snoozing = false;
  }
  
}

void checkBlindAlarm() {
  // second=0, minute=0, hour=0, weekday=7
  // alarmMin = 0, alarmHour = 0;
  int blindMin = minuteA - blindPrevTime;
  int blindHour = hourA;
  
  if (blindMin < 0) {
    blindMin = 60 + (blindMin);
    blindHour = blindHour - 1;
    if (blindHour < 0) blindHour = 23;
  }
  
}


void onSnoozePress() {
  snoozing = true;
  digitalWrite(speakerPin, LOW);
  soundAlarm = 0;
  int tempMin = minute + snoozeTime;
  int tempHour = hour;
  
  if (tempMin >= 60) {
    tempMin = tempMin - 60;
    tempHour = tempHour + 1;
    if (tempHour >= 24) tempHour = 0;
  }
  minuteSn = tempMin;
  hourSn = tempHour;
  //Serial.print("Snoozing: ");
  //Serial.print(hourSn);
  //Serial.print(" : ");
  //Serial.println(minuteSn);
}



void checkButtons() {
  
  if (digitalRead(alarmPin) == LOW && alarmPressed == false) {
    alarmOn = !alarmOn;
    //Serial.println("Alarm button press");
    if (alarmOn == true) {
      digitalWrite(ledPin, HIGH);
      printAlarmSymbol(true);
    } else {
      digitalWrite(ledPin, LOW);
      printAlarmSymbol(false);
    }
    alarmPressed = true;
  }
  if (digitalRead(alarmPin) == HIGH) alarmPressed = false;
  
  if (digitalRead(snoozePin) == LOW && snoozePressed == false) {
    snoozeTracker = millis();
    snoozePressed = true;
    //Serial.println("Snooze button pressed Start");
    //Serial.println("Snooze button press");
    //if (soundAlarm > 0) onSnoozePress();
  }
  //if (millis() - snoozeTracker > 300 && snoozePressed == true) {
    //if (soundAlarm > 0) onSnoozePress();
    //Serial.println("Snooze button pressed");
  //}
  //Serial.println(digitalRead(snoozePin));
  
  if (digitalRead(snoozePin) == HIGH) snoozePressed = false;
  
  // check the time setting clock buttons //
  if (analogRead(minutePin) >= 1000 && minPressed == false) {
    minute++;
    if (minute >= 60) minute = 0;
    minPressed = true;
  }
  if (analogRead(minutePin) <= 10) minPressed = false;

  if (digitalRead(hourPin) == HIGH && hourPressed == false) {
    hour++;
    if (hour >= 24) hour = 0;
    hourPressed = true;
  }
  if (digitalRead(hourPin) == LOW) hourPressed = false;
  
  
  // check the alarm setting buttons //
  if (analogRead(minutePinA) >= 1000 && minPressedA == false) {
    minuteA++;
    if (minuteA >= 60) minuteA = 0;
    minPressedA = true;
  }
  if (analogRead(minutePinA) <= 10) minPressedA = false;

  if (analogRead(hourPinA) >= 1000 && hourPressedA == false) {
    hourA++;
    if (hourA >= 24) hourA = 0;
    hourPressedA = true;
  }
  if (analogRead(hourPinA) <= 10) hourPressedA = false;
  
  // check if we are setting the time //
  if (minPressed || hourPressed) {
    settingTime = true;
    snoozing = false;
  } else {
    settingTime = false;
  }
  // check if you are setting the alarm tizzime //
  if (minPressedA || hourPressedA) {
    lastSetA = millis();
    settingTimeA = true;
    snoozing = false;
  } else {
    settingTimeA = false;
  }
}

// send the data wirelessly //

void sendData(int num) {
  // N i C k
  
  Serial.print(78, BYTE);
  Serial.print(98, BYTE);
  
  Serial.print(num, BYTE);
   //rfSerial.print(78, BYTE);
   //rfSerial.print(98, BYTE);
   
   //rfSerial.print(num, BYTE);
   
   //rfSerial.print(67, BYTE);
   //rfSerial.print(97, BYTE);
   //Serial.print("sending: ");
   //Serial.println(num);
}



